using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace DXSpotterClient
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
        }

        protected int m_nResult = -1;

        private void OnLoad(object sender, EventArgs e)
        {
            this.EnumSpotters();
        }

        [System.Security.Permissions.PermissionSet(System.Security.Permissions.SecurityAction.Demand, Name = "FullTrust")]
        protected override void WndProc(ref Message m)
        {
            // Listen for operating system messages.
            switch(m.Msg)
            {
                case Win32.WM_COPYDATA:
                    {
                        Win32.COPYDATASTRUCT cd = new Win32.COPYDATASTRUCT();
                        cd = (Win32.COPYDATASTRUCT)Marshal.PtrToStructure(m.LParam, typeof(Win32.COPYDATASTRUCT)); 
                        m_nResult = cd.dwData;
                        
                        int nLen = cd.cbData;
                        if (nLen > 0)
                        {
                            String strBuffer = String.Empty;
                            strBuffer = Marshal.PtrToStringAnsi((IntPtr)cd.lpData).Substring(0, cd.cbData);
                            
                            this.richTextBoxInfo.Text = strBuffer;
                        }
                    }
                    break;
            }
            base.WndProc(ref m);
        }

        protected void EnumSpotters()
        {
            this.listBoxDXS.Items.Clear();

            IntPtr hWnd = Win32.FindWindow(null, null); // desktop

            String strCall;
            StringBuilder strBuffer = new StringBuilder(260);

            while (!(hWnd.Equals(IntPtr.Zero)))
	        {
		        hWnd = Win32.GetNextWindow(hWnd,Win32.GW_HWNDNEXT);

                strBuffer.Length = 0;
		        if(!hWnd.Equals(IntPtr.Zero))
		        {
                    Win32.GetWindowTextA(hWnd, strBuffer, 260);
                    if(strBuffer.Length > 0)
                    {
                        strCall = strBuffer.ToString();
                        if(strCall.Contains("DXSpotter:") && strCall.Contains("cluster"))
                        {
                            strCall = strCall.Substring(strCall.IndexOf(':') + 1);
                            strCall.Trim();

                            String [] strCalls = strCall.Split(' ');

                            this.listBoxDXS.Items.Add(strCalls[1]);
                        }
                    }
		        }
	        }

            if (listBoxDXS.Items.Count > 0) listBoxDXS.SelectedIndex = 0;
        }

        public int VarPtr(object e)
        {
            GCHandle GC = GCHandle.Alloc(e, GCHandleType.Pinned);
            int gc = GC.AddrOfPinnedObject().ToInt32();
            GC.Free();
            return gc;
        }

        protected void SendMessageToSpotter(String strMessage)
        {
	        int nSelected = this.listBoxDXS.SelectedIndex;

	        if(nSelected!=-1)
	        {
		        String strCall;
		        strCall = this.listBoxDXS.Text;

		        strCall = "DXSpotter: "+strCall+" cluster";

		        IntPtr hWndTarget = Win32.FindWindow("#32770",strCall);

                if (!hWndTarget.Equals(IntPtr.Zero))
                {
                    Win32.COPYDATASTRUCT cd;
                    cd.lpData = VarPtr(strMessage);
                    cd.dwData = this.Handle.ToInt32();
                    cd.cbData = strMessage.Length * 2; // unicode string length

                    Win32.SendMessage(hWndTarget, Win32.WM_COPYDATA, 0, VarPtr(cd));
                }
	        }
	        else
                MessageBox.Show("DXSpotter is not running.");
        }

        private void OnButtonOK(object sender, EventArgs e)
        {
            this.Close();
        }

        private void OnButtonRefresh(object sender, EventArgs e)
        {
            this.EnumSpotters();
        }

        private void OnButtonClose(object sender, EventArgs e)
        {
	        int nSelected = this.listBoxDXS.SelectedIndex;

	        if(nSelected!=-1)
	        {
		        String strCall;
		        strCall = this.listBoxDXS.Text;

		        strCall = "DXSpotter: "+strCall+" cluster";

		        this.SendMessageToSpotter("IsConnected");

		        if(m_nResult == 0)
		        {
			        IntPtr hWndTarget = Win32.FindWindow("#32770",strCall);

                    if (!hWndTarget.Equals(IntPtr.Zero))
				        Win32.SendMessage(hWndTarget, Win32.WM_SYSCOMMAND, Win32.SC_CLOSE, 0);
		        }
		        else 
			        MessageBox.Show("DXSpotter is still connected. Please disconnect it from the cluster first.");
	        }
	        else 
		        MessageBox.Show("DXSpotter is not running.");
        }

        private void OnButtonConnect(object sender, EventArgs e)
        {
	        int nSelected = this.listBoxDXS.SelectedIndex;

	        if(nSelected!=-1)
	        {
		        String strCall, strTarget;
		        strCall = this.listBoxDXS.Text;

                strTarget = this.textBoxCluster.Text;
                strTarget.Trim();

		        if(strTarget.Length>2)
		        {
			        strCall = this.listBoxDXS.Text;

			        strCall = "DXSpotter: "+strCall+" cluster";

			        SendMessageToSpotter("Connect:"+strTarget);
		        }
		        else
		        {
			        MessageBox.Show("Please select valid cluster callsign.");
			        this.textBoxCluster.Focus();
		        }
	        }
            else
                MessageBox.Show("DXSpotter is not running.");
        }

        private void OnButtonDisconnect(object sender, EventArgs e)
        {
	        int nSelected = this.listBoxDXS.SelectedIndex;

	        if(nSelected!=-1)
        		SendMessageToSpotter("Disconnect");
            else
                MessageBox.Show("DXSpotter is not running.");
        }

        private void OnButtonGetAllText(object sender, EventArgs e)
        {
            int nSelected = this.listBoxDXS.SelectedIndex;

            if (nSelected != -1)
                SendMessageToSpotter("GetScreenText");
            else
                MessageBox.Show("DXSpotter is not running.");
        }

        private void OnButtonGetAllSpots(object sender, EventArgs e)
        {
            int nSelected = this.listBoxDXS.SelectedIndex;

            if (nSelected != -1)
                SendMessageToSpotter("GetSpots");
            else
                MessageBox.Show("DXSpotter is not running.");
        }

        private void OnButtonGetAnnounces(object sender, EventArgs e)
        {
            int nSelected = this.listBoxDXS.SelectedIndex;

            if (nSelected != -1)
                SendMessageToSpotter("GetAnnounces");
            else
                MessageBox.Show("DXSpotter is not running.");
        }

        private void OnButtonGetLast5DX(object sender, EventArgs e)
        {
            int nSelected = this.listBoxDXS.SelectedIndex;

            if (nSelected != -1)
                SendMessageToSpotter("GetLast5DX");
            else
                MessageBox.Show("DXSpotter is not running.");

        }

        private void OnButtonGetLast10DX(object sender, EventArgs e)
        {
            int nSelected = this.listBoxDXS.SelectedIndex;

            if (nSelected != -1)
                SendMessageToSpotter("GetLast10DX");
            else
                MessageBox.Show("DXSpotter is not running.");
        }

        private void OnButtonSend(object sender, EventArgs e)
        {
            int nSelected = this.listBoxDXS.SelectedIndex;

            if (nSelected != -1)
            {
                String strMessage;
                strMessage = this.textBoxMessage.Text;

                SendMessageToSpotter("SendMessage:" + strMessage);
            }
        }
    }
}