Imports System
Imports System.io
Imports System.Runtime.InteropServices

Public Class MainForm

#Region "CopyData structure"

    <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Ansi)> _
    Private Structure CopyData
        Public dwData As IntPtr
        Public cbData As Integer
        Public lpData As IntPtr
    End Structure

#End Region


#Region "Win32 API functions"

    Private Declare Function FindWindow Lib "user32" Alias "FindWindowA" _
        (ByVal lpClassName As String, ByVal lpWindowName As String) As IntPtr

    Private Declare Function GetNextWindow Lib "user32" Alias "GetWindow" _
        (ByVal hWnd As IntPtr, ByVal wFlag As Integer) As IntPtr

    Private Declare Function GetWindowTextA Lib "user32" Alias "GetWindowTextA" _
        (ByVal hWnd As IntPtr, ByVal lpWindowName As String, ByVal nCount As Integer) As Integer

    Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" _
        (ByVal hwnd As IntPtr, ByVal wMsg As IntPtr, ByVal wParam As Integer, _
        ByVal lParam As Integer) As Integer

#End Region

#Region "Win32 API constants"

    Public Const WM_SYSCOMMAND = &H112
    Public Const WM_COPYDATA = &H4A
    Public Const SC_CLOSE = &HF060
    Public Const GW_HWNDNEXT = 2&

#End Region

    Protected m_nResult = -1

    Public Sub New()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.

    End Sub

    Protected Overrides Sub OnLoad(ByVal e As System.EventArgs)
        Me.EnumSpotters()
        MyBase.OnLoad(e)
    End Sub

    Protected Overrides Sub WndProc(ByRef m As System.Windows.Forms.Message)
        Select Case m.Msg
            Case WM_COPYDATA
                Dim cd As CopyData
                Dim nLen As Integer
                Dim strBuffer As String

                cd = New CopyData
                cd = Marshal.PtrToStructure(m.LParam, GetType(CopyData))
                m_nResult = cd.dwData

                nLen = cd.cbData
                If (nLen > 0) Then
                    strBuffer = Marshal.PtrToStringAnsi(cd.lpData).Substring(0, cd.cbData)
                    Me.richTextBoxInfo.Text = strBuffer
                End If
        End Select
        MyBase.WndProc(m)
    End Sub

    Protected Sub EnumSpotters()
        Me.listBoxDXS.Items.Clear()
        Dim hWnd As IntPtr
        Dim strBuffer As String
        Dim strCall As String
        Dim strCalls As String()

        hWnd = FindWindow(vbNullString, vbNullString)

        While (hWnd.ToInt32 <> 0)

            strBuffer = New String(CType(" ", Char), 260)
            hWnd = GetNextWindow(hWnd, GW_HWNDNEXT)
            If hWnd.ToInt32 <> 0 Then
                GetWindowTextA(hWnd, strBuffer, 260)
                If strBuffer.Length > 0 Then
                    If strBuffer.Contains("DXSpotter:") And strBuffer.Contains("cluster") Then
                        strCall = strBuffer
                        strCall = strCall.Substring(strCall.IndexOf(CType(":", Char)) + 1)
                        strCall.Trim()

                        strCalls = strCall.Split(CType(" ", Char))

                        Me.listBoxDXS.Items.Add(strCalls(1))
                        If listBoxDXS.Items.Count > 0 Then Me.listBoxDXS.SelectedIndex = 0
                    End If
                End If
            End If

        End While

    End Sub

    Public Function VarPtr(ByRef e As Object) As Integer

        Dim gch As GCHandle
        Dim pObject As Integer

        gch = New GCHandle
        gch = GCHandle.Alloc(e, GCHandleType.Pinned)
        pObject = gch.AddrOfPinnedObject().ToInt32

        gch.Free()

        Return pObject

    End Function

    Protected Sub SendMessageToSpotter(ByVal strMessage As String)
        Dim nSelected As Integer
        Dim hWndTarget As IntPtr
        Dim strCall As String
        Dim data As CopyData

        nSelected = Me.listBoxDXS.SelectedIndex
        If nSelected <> -1 Then
            strCall = Me.listBoxDXS.Text
            strCall = "DXSpotter: " + strCall + " cluster"

            hWndTarget = FindWindow("#32770", strCall)

            If (Not hWndTarget.Equals(IntPtr.Zero)) Then
                data = New CopyData()
                data.lpData = VarPtr(strMessage)
                data.dwData = Me.Handle.ToInt32
                data.cbData = strMessage.Length * 2

                SendMessage(hWndTarget, WM_COPYDATA, 0, VarPtr(data))
            End If
        Else
            MessageBox.Show("DXSpotter is not running.")
        End If

    End Sub

    Private Sub OnButtonOK(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonOK.Click
        Me.Close()
    End Sub

    Private Sub OnButtonRefresh(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonRefresh.Click
        Me.EnumSpotters()
    End Sub

    Private Sub OnButtonClose(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonClose.Click
        Dim nSelected As Integer
        Dim hWndTarget As IntPtr
        Dim strCall As String

        nSelected = Me.listBoxDXS.SelectedIndex
        If nSelected <> -1 Then
            strCall = Me.listBoxDXS.Text
            strCall = "DXSpotter: " + strCall + " cluster"

            Me.SendMessageToSpotter("IsConnected")

            If m_nResult = 0 Then
                hWndTarget = FindWindow("#32770", strCall)

                If (Not hWndTarget.Equals(IntPtr.Zero)) Then
                    SendMessage(hWndTarget, WM_SYSCOMMAND, SC_CLOSE, 0)
                End If
            Else
                MessageBox.Show("DXSpotter is still connected. Please disconnect it from the cluster first.")
            End If
        Else
            MessageBox.Show("DXSpotter is not running.")
        End If
    End Sub

    Private Sub OnButtonConnect(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonConnect.Click
        Dim nSelected As Integer
        Dim strCall As String
        Dim strTarget As String

        nSelected = Me.listBoxDXS.SelectedIndex
        If nSelected <> -1 Then
            strCall = Me.listBoxDXS.Text
            strCall = "DXSpotter: " + strCall + " cluster"
            strTarget = Me.textBoxCluster.Text

            If strTarget.Length > 0 Then
                Me.SendMessageToSpotter("Connect:" + strTarget)
            Else
                MessageBox.Show("Please select valid cluster callsign.")
            End If
        Else
            MessageBox.Show("DXSpotter is not running.")
        End If
    End Sub

    Private Sub OnButtonDisconnect(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonDisconnect.Click
        Dim nSelected As Integer

        nSelected = Me.listBoxDXS.SelectedIndex
        If nSelected <> -1 Then
            Me.SendMessageToSpotter("Disconnect")
        Else
            MessageBox.Show("DXSpotter is not running.")
        End If
    End Sub

    Private Sub OnButtonGetAllText(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonGetText.Click
        Dim nSelected As Integer

        nSelected = Me.listBoxDXS.SelectedIndex
        If nSelected <> -1 Then
            Me.SendMessageToSpotter("GetScreenText")
        Else
            MessageBox.Show("DXSpotter is not running.")
        End If
    End Sub

    Private Sub OnButtonGetAllSpots(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonGetSpots.Click
        Dim nSelected As Integer

        nSelected = Me.listBoxDXS.SelectedIndex
        If nSelected <> -1 Then
            Me.SendMessageToSpotter("GetSpots")
        Else
            MessageBox.Show("DXSpotter is not running.")
        End If
    End Sub

    Private Sub OnButtonGetAnnounces(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonGetAnn.Click
        Dim nSelected As Integer

        nSelected = Me.listBoxDXS.SelectedIndex
        If nSelected <> -1 Then
            Me.SendMessageToSpotter("GetAnnounces")
        Else
            MessageBox.Show("DXSpotter is not running.")
        End If
    End Sub

    Private Sub OnButtonGetLast5DX(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonGetLast5.Click
        Dim nSelected As Integer

        nSelected = Me.listBoxDXS.SelectedIndex
        If nSelected <> -1 Then
            Me.SendMessageToSpotter("GetLast5DX")
        Else
            MessageBox.Show("DXSpotter is not running.")
        End If
    End Sub

    Private Sub OnButtonGetLast10DX(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonGetLast10.Click
        Dim nSelected As Integer

        nSelected = Me.listBoxDXS.SelectedIndex
        If nSelected <> -1 Then
            Me.SendMessageToSpotter("GetLast10DX")
        Else
            MessageBox.Show("DXSpotter is not running.")
        End If
    End Sub

    Private Sub OnButtonSend(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonSend.Click
        Dim nSelected As Integer
        Dim strMessage As String

        nSelected = Me.listBoxDXS.SelectedIndex
        If nSelected <> -1 Then
            strMessage = Me.textBoxMessage.Text
            Me.SendMessageToSpotter("SendMessage:" + strMessage)
        Else
            MessageBox.Show("DXSpotter is not running.")
        End If
    End Sub
End Class
